/*
 * SPDX-FileCopyrightText: 2020 Piyush Aggarwal <piyushaggarwal002@gmail.com>
 *
 * SPDX-License-Identifier: CC0-1.0 OR LicenseRef-KDE-Accepted-GPL
 */

#ifndef SYSTRAY_ACTIONS_H
#define SYSTRAY_ACTIONS_H
#include <battery_action.h>
#include <connectivity_action.h>
#endif // SYSTRAY_ACTIONS_H
